/**
 * NOTE: this error is only handled gracefully for plugins that are hosted as dedicated transformation APIs.
 */
export class TransformationDisplayableError extends Error {
  /**
   * @param message Human readable message.
   * @param errorCode Machine readable error code.
   * @param isClientError False if a bug, true if client has any recourse.
   * @param httpStatusCode Status code, if the application supports custom status codes.
   *                   API supports custom status codes. Transform and Edge do not.
   */
  constructor(
    message: string,
    readonly errorCode: string,
    readonly isClientError: boolean,

    // The properties 'status' and 'statusCode' are somehow intercepted and defiled by Koa or TSOA (it sets them to 500
    // if undefined) -- so we intentionally choose a property name that doesn't get fiddled with by these frameworks.
    readonly httpStatusCode?: number,

    readonly details?: {
      [key: string]: unknown;
    }
  ) {
    super(message);
    Object.setPrototypeOf(this, TransformationDisplayableError.prototype);
    this.name = "TransformationDisplayableError";
  }
}
